<?php
/*

Copyright: 2011 MediaSlurp
License: Apache 2.0

Plugin Name: UserVoice Idea List Widget

Plugin URI: http://mediaslurp.com/download/uservoice-idea-list-widget/

Description: A widget that displays the most popular ideas within a specific UserVoice forum. To configure, edit the widget via Appearance>>Widgets>>UserVoice Idea List Widget.

Version: 1.3

Author: Maintained by TJ Downes of MediaSlurp. Rob Knight. Originally produced by Shane & Peter, Inc.
Author URI: http://mediaslurp.com/download/uservoice-idea-list-widget/

Maintained by: MediaSlurp development team - TJ Downes, Vicky Ryder and Ben Farrell
Maintainer URI: http://mediaslurp.com/download/uservoice-idea-list-widget/

History:
1.3
	* Updated for the most recent UV API (version 1). This should have gone into 1.2 but there were bugs that we weren't certain we would have fixed on time
	* Now requires API KEY!!!!
	* account name has been updated and you should only require the account name, not the full uservoice subdomain (mediaslurp instead of mediaslurp.uservoice.com)

1.2
	* Updated widget to fix URL, wehich was broken and did not allow plugin to function
	* reformatted code for readability
	* updated some text elements for usability, in management console
	* updated classes on text inputs in widget editor so they would span entire width of the widget editor
*/

class uservoice_suggestions_widget 
{
    var $id = "uservoice-suggestions";
	var $name = "UserVoice";
	var $classname = "uservoice_suggestions_widget";
	var $optionsname = "uservoice_suggestions_widget_option";
	var $description = "UserVoice suggestions widget";

    // Show the widget
    function widget($args) 
	{
        extract($args);

        $options = get_option($this->optionsname);

        $title = $options[$this->id]['title'];

        $account = $options[$this->id]['account'];

        $forum_id = $options[$this->id]['forum_id'];

        $count = $options[$this->id]['count'];
        
        $api_key = $options[$this->id]['api_key'];

        echo $before_widget;        
        echo $before_title . $title . $after_title;
		
        //Run function to return HTML with suggestions	
        echo $this->uservoice_get_suggestions($account, $forum_id, $count, $api_key);    
        echo $after_widget;
    }

    // Widget Panel
    function control() 
	{
        $title = $this->id.'-title';
        $account = $this->id.'-account';
        $forum_id = $this->id.'-forum_id';
        $count = $this->id.'-count';
        $api_key = $this->id.'-api_key';
        $submit = $this->id.'-submit';

        // Update DB if new data
        if ($_POST[$submit]) 
		{
            $options[$this->id]['title'] = htmlentities(stripslashes(trim($_POST[$title])));
            $options[$this->id]['account'] = htmlentities(stripslashes(trim($_POST[$account])));
			$options[$this->id]['forum_id'] = htmlentities(stripslashes(trim($_POST[$forum_id])));
			$options[$this->id]['count'] = htmlentities(stripslashes(trim($_POST[$count])));
			$options[$this->id]['api_key'] = htmlentities(stripslashes(trim($_POST[$api_key])));

			update_option($this->optionsname, $options);
        }

        // Get Data from Options
		$options = get_option($this->optionsname);		

		if ( !is_array($options) ) 
		{
			$options = array();

			$options[$this->id] = array(
				'title' => "Community Suggestions",
				'account' => "santacruz",
				'forum_id' => 18105,
				'count' => 5,
				'api_key' => ''
			);
		}

		// Display the widget control panel
		echo '<label for="'.$title.'">Widget title:<br /><input type="text" value="'.$options[$this->id]['title'].'" name="'.$title.'" class="widefat" /></label><br />';

		echo '<label for="'.$account.'">Account Name:<br /><br /><input type="text" value="'.$options[$this->id]['account'].'" name="'.$account.'" class="widefat" /></label><br />';

		echo '<label for="'.$forum_id.'">Forum ID: <br /><input type="text" value="'.$options[$this->id]['forum_id'].'" name="'.$forum_id.'" class="widefat" /></label><br />';

		echo '<label for="'.$count.'">Count: <br /><input type="text" value="'.$options[$this->id]['count'].'" name="'.$count.'" /></label><br />';
		
		echo '<label for="'.api_key.'">API Key: <br /><input type="text" value="'.$options[$this->id]['api_key'].'" name="'.$api_key.'" class="widefat" /></label>';

		echo '<input type="hidden" id="'.$submit.'" name="'.$submit.'" value="1" class="widefat" /></p>';
	}
	
    // Initialize the widget
		function init() 
		{
			wp_register_sidebar_widget(
				$this->id,
				$this->name,
				array(&$this, 'widget'),
				array('classname' => $this->classname, 'description' => $this->description),
				array( 'number' => -1 )
			);

			wp_register_widget_control(
				$this->id,
				$this->name,
				array(&$this, 'control')
			);
		}
		
    /**

     *  uservoice_get_suggestions

     *  Loads the XML from the UserVoice API and puts it into <ul> for display in the widget

     *  @return HTML for widget display

     *  @param $account = The UserVoice username 

     *  @param $forum_id = The numeric representation of the forum

     **/    

    function uservoice_get_suggestions($account, $forum_id, $count, $api_key) 
	{
        $base_url = "http://".$account.".uservoice.com/api/v1/forums/".$forum_id."/suggestions.xml?per_page=".$count."&sort=votes&client=".$api_key;
        
        // Load the XML with a cURL request
        $xml = $this->load_xml($base_url);
        
        $html = '<div id="uv_box">';
        $suggestions = $xml->suggestions[0]->suggestion;
        $suggestionCount = count($suggestions);
        
        if($suggestionCount)
        {
	        // init the list, then populate with the suggestions
	        $html .= '<ul id="uv_suggestion_list">';
	        
	        for ($i=0; $i < $suggestionCount; $i++) 
			{ 
	            // Show 'comment' or 'comments' depending on the number of comments
	            if ($suggestions[$i]->comments_count == 1) 
				{ 
					$comment_text = "&nbsp;comment"; 
				}
	            else 
				{
					$comment_text = "&nbsp;comments";
				}
	
	            $html .= '<li class="uv_suggestion">';
	
	            $html .= '<span class="uv_votes"><span class="uv_num_votes">'.$suggestions[$i]->vote_count.'</span> votes</span> ';
	
	            $html .= '<a href="'.$suggestions[$i]->url.'">'.$suggestions[$i]->title.'</a> ';
	
	            // Show number of comments if there are any
	
	            if ($suggestions[$i]->comments_count > 0) 
				{
	                $html .= '<br /><a class="uv_comments" href="'.$suggestions[$i]->url.'">'.$suggestions[$i]->comments_count.$comment_text.'</a>';
	            }
				
	            $html .= '</li>';
	        }

       		$html .= '</ul>';
        }
        else
        {
        	$html .= 'No current feedback<br />';
        }
        
        $html .= '<a id="uv_credit" href="http://uservoice.com">Powered By UserVoice.</a>';
        $html .= '</div>';
        
        return $html;
    }

    /**

     * load_xml function

     * @return XML string of UserVoice suggestions

     * @param $url Where to get the XML from 

     **/

    function load_xml($url) 
	{
        $ch = curl_init($url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_HEADER, 0);
		
        $data = simplexml_load_string(curl_exec($ch));
        curl_close($ch);
        return $data;
    }
	
    /**

     *  add_css function

     *  Adds <link> tag to add CSS to WP

     **/

    function add_css() 
	{
        echo '<link rel="stylesheet" href="'.get_bloginfo('wpurl').'/wp-content/plugins/uservoice-idea-list-widget/uv_styles.css" type="text/css" />';
    }

} // end Class


$o = new uservoice_suggestions_widget();

add_action('plugins_loaded', array($o,'init'));

add_action('wp_head', array($o,'add_css'));